@@ -1,5 +1,5 @@
 from abc import ABC
-from typing import Optional
+from typing import Optional, Dict, List, Any
 
 from pydantic import BaseModel
 
@@ -11,3 +11,5 @@ class BaseData(BaseModel, ABC):
     """
     external_id: Optional[str] = None
     uid: Optional[str] = None
+    media_attributes: Optional[Dict[str, Any]] = None
+    metadata: Optional[List[Dict[str, Any]]] = None
