@@ -41,7 +41,11 @@ def results_query_part(entity):
         entity (type): The entity which needs fetching.
     """
     # Query for fields
-    fields = [field.graphql_name for field in entity.fields()]
+    fields = [
+        field.result_subquery
+        if field.result_subquery is not None else field.graphql_name
+        for field in entity.fields()
+    ]
 
     # Query for cached relationships
     fields.extend([
