@@ -93,6 +93,22 @@ class Project(DbObject, Updateable, Deletable):
         Dataset = "Dataset"
 
     def update(self, **kwargs):
+        """ Updates this project with the specified attributes
+
+        Args:
+            kwargs: a dictionary containing attributes to be upserted
+
+        Note that the quality setting cannot be changed after a project has been created. The quality mode
+            for a project is inferred through the following attributes:
+            Benchmark:
+                auto_audit_number_of_labels = 1
+                auto_audit_percentage = 1.0
+            Consensus:
+                auto_audit_number_of_labels > 1
+                auto_audit_percentage <= 1.0
+            Attempting to switch between benchmark and consensus modes is an invalid operation and will result
+            in an error.
+        """
         mode: Optional[Project.QueueMode] = kwargs.pop("queue_mode", None)
         if mode:
             self._update_queue_mode(mode)
